@@ -4,7 +4,7 @@
 #
 # Author: Just van den Broecke
 #
-from stetl.component import Attr
+from stetl.component import Config
 from stetl.input import Input
 from stetl.util import Util, etree
 from stetl.packet import FORMAT
@@ -13,44 +13,90 @@ import csv
 log = Util.get_log('fileinput')
 
 
-
 class FileInput(Input):
     """
     Abstract base class for specific FileInputs, use derived classes.
     """
 
     # Start attribute config meta
-    cfg_file_path = Attr(str, True, None,
-    "path to file or files: can be a dir or files or even multiple, comma separated")
+    # Applying Decorator pattern with the Config class to provide
+    # read-only config values from the configured properties.
+    
+    @Config(ptype=str, default=None, required=False)
+    def file_path(self):
+        """
+        Path to file or files or URLs: can be a dir or files or URLs
+        or even multiple, comma separated. For URLs only JSON is supported now.
 
-    cfg_filename_pattern = Attr(str, False, '*.[gxGX][mM][lL]',
-    "filename pattern according to Python glob.glob")
+        Required: True
+
+        Default: None
+        """
+        pass
+
+
+    @Config(ptype=str, default='*.[gxGX][mM][lL]', required=False)
+    def filename_pattern(self):
+        """
+        Filename pattern according to Python ``glob.glob`` for example:
+        '\*.[gxGX][mM][lL]'
+
+        Required: False
+
+        Default: '\*.[gxGX][mM][lL]'
+        """
+        pass
+
+    @Config(ptype=bool, default=False, required=False)
+    def depth_search(self):
+        """
+        Should we recurse into sub-directories to find files?
+
+        Required: False
+
+        Default: False
+        """
+        pass
 
-    cfg_depth_search = Attr(bool, False, False, "Recurse into directories ?")
     # End attribute config meta
 
     def __init__(self, configdict, section, produces):
         Input.__init__(self, configdict, section, produces)
 
-        # path to file or files: can be a dir or files or even multiple, comma separated
-        self.file_path = self.cfg.get('file_path')
-
-        # The filename pattern according to Python glob.glob
-        self.filename_pattern = self.cfg.get('filename_pattern', FileInput.cfg_filename_pattern.default)
-
-        # Recurse into directories ?
-        self.depth_search = self.cfg.get_bool('depth_search', FileInput.cfg_depth_search.default)
-
         # Create the list of files to be used as input
         self.file_list = Util.make_file_list(self.file_path, None, self.filename_pattern, self.depth_search)
         log.info("file_list=%s" % str(self.file_list))
+        if not len(self.file_list):
+            raise Exception('File list is empty!!')
+
+        self.file_list_done = []
 
     def read(self, packet):
+        if not len(self.file_list):
+            return packet
+
+        file_path = self.file_list.pop(0)
+
+        log.info("Read/parse for start for file=%s...." % file_path)
+        packet.data = self.read_file(file_path)
+        log.info("Read/parse ok for file=%s" % file_path)
+
+        # One-time read: we're all done
+        packet.set_end_of_doc()
+        if not len(self.file_list):
+            log.info("all files done")
+            packet.set_end_of_stream()
+
+        self.file_list_done.append(file_path)
+        return packet
+
+    def read_file(self, file_path):
         """
         Override in subclass.
         """
         pass
 
+
 class StringFileInput(FileInput):
     """
     Reads and produces file as String.
@@ -59,23 +105,28 @@ class StringFileInput(FileInput):
     """
 
     # Start attribute config meta
-    cfg_format_args = Attr(str, False, None,
-    """
-    formatting of content according to Python String.format()
-    Input file should have substitutable values like {schema} {foo}
-    format_args should be of the form format_args = schema:test foo:bar
-    """)
+    @Config(ptype=str, default=None, required=False)
+    def format_args(self):
+        """
+        Formatting of content according to Python String.format()
+        Input file should have substitutable values like {schema} {foo}
+        format_args should be of the form ``format_args = schema:test foo:bar``
+
+        Required: False
+
+        Default: None
+        """
+        pass
+
     # End attribute config meta
 
     def __init__(self, configdict, section):
         FileInput.__init__(self, configdict, section, produces=FORMAT.string)
-        self.file_list_done = []
         self.file = None
 
         # Optional formatting of content according to Python String.format()
         # Input file should have substitutable values like {schema} {foo}
         # format_args should be of the form format_args = schema:test foo:bar
-        self.format_args = self.cfg.get('format_args')
 
         if self.format_args:
             # Convert string to dict: http://stackoverflow.com/a/1248990
@@ -122,84 +173,18 @@ class XmlFileInput(FileInput):
     # Constructor
     def __init__(self, configdict, section):
         FileInput.__init__(self, configdict, section, produces=FORMAT.etree_doc)
-        self.file_list_done = []
 
-    def read(self, packet):
-        if not len(self.file_list):
-            return packet
-
-        file_path = self.file_list.pop(0)
+    def read_file(self, file_path):
         # One-time read/parse only
+        data = None
         try:
-            packet.data = etree.parse(file_path)
-            log.info("file read and parsed OK : %s" % file_path)
+            data = etree.parse(file_path)
         except Exception, e:
             log.info("file read and parsed NOT OK : %s" % file_path)
 
-        # One-time read: we're all done
-        packet.set_end_of_doc()
-        if not len(self.file_list):
-            log.info("all files done")
-            packet.set_end_of_stream()
-
-        self.file_list_done.append(file_path)
-        return packet
+        return data
 
 
-class XmlLineStreamerFileInput(FileInput):
-    """
-    DEPRECATED Streams lines from an XML file(s)
-    NB assumed is that lines in the file have newlines !!
-    DEPRECATED better is to use XmlElementStreamerFileInput for GML features.
-
-    produces=FORMAT.xml_line_stream
-    """
-
-    # Constructor
-    def __init__(self, configdict, section):
-        FileInput.__init__(self, configdict, section, produces=FORMAT.xml_line_stream)
-        self.file_list_done = []
-        self.file = None
-
-    def read(self, packet):
-        # No more files left and done with current file ?
-        if not len(self.file_list) and self.file is None:
-            packet.set_end_of_stream()
-            log.info("EOF file list")
-            return packet
-
-        # Done with current file or first file ?
-        if self.file is None:
-            self.cur_file_path = self.file_list.pop(0)
-            self.file = open(self.cur_file_path, 'r')
-            log.info("file opened : %s" % self.cur_file_path)
-
-        if packet.is_end_of_stream():
-            return packet
-
-        # Assume valid line
-        line = self.file.readline()
-
-        # EOF reached ?
-        if not line or line == '':
-            packet.data = None
-
-            packet.set_end_of_doc()
-            log.info("EOF file")
-            if self.cur_file_path is not None:
-                self.file_list_done.append(self.cur_file_path)
-                self.cur_file_path = None
-                if not len(self.file_list):
-                    # No more files left: end of stream reached
-                    packet.set_end_of_stream()
-                    log.info("EOF file list")
-
-            self.file = None
-
-            return packet
-
-        packet.data = line.decode('utf-8')
-        return packet
 
 class XmlElementStreamerFileInput(FileInput):
     """
@@ -207,32 +192,44 @@ class XmlElementStreamerFileInput(FileInput):
     Parsing is streaming (no internal DOM buildup) so any file size can be handled.
     Use this class for your big GML files!
 
-    produces=FORMAT.etree_element_stream
+    produces=FORMAT.etree_element
     """
 
     # Start attribute config meta
-    cfg_element_tags = Attr(str, True, None,
-    """
-    comma-separated string of XML (feature) element tagnames of the elements that should be extracted
-    and added to the output element stream.
-    """)
+    @Config(ptype=list, default=None, required=True)
+    def element_tags(self):
+        """
+        Comma-separated string of XML (feature) element tag names of the elements that should be extracted
+        and added to the output element stream.
+
+        Required: True
+
+        Default: None
+        """
+        pass
+
+    @Config(ptype=bool, default=False, required=False)
+    def strip_namespaces(self):
+        """
+        should namespaces be removed from the input document and thus not be present in the output element stream?
+
+        Required: False
+
+        Default: False
+        """
+        pass
 
-    cfg_strip_namespaces = Attr(bool, False, False,
-    """
-    should namespaces be removed from the input document and thus not be present in the output element stream?
-    """)
     # End attribute config meta
 
     # Constructor
     def __init__(self, configdict, section):
-        FileInput.__init__(self, configdict, section, produces=FORMAT.etree_element_stream)
-        self.element_tags = self.cfg.get('element_tags').split(',')
+        FileInput.__init__(self, configdict, section, produces=FORMAT.etree_element)
         self.file_list_done = []
-        self.strip_namespaces = self.cfg.get('strip_namespaces', XmlElementStreamerFileInput.cfg_element_tags.default)
         self.context = None
         self.root = None
         self.cur_file_path = None
         self.elem_count = 0
+        log.info("Element tags to be matched: %s" % self.element_tags)
 
     def read(self, packet):
         event = None
@@ -300,18 +297,96 @@ class XmlElementStreamerFileInput(FileInput):
 
         return packet
 
-
-class CsvFileInput(FileInput):
+class XmlLineStreamerFileInput(FileInput):
     """
-    Parse CSV file into stream of records (dict structures).
-    NB raw version: CSV needs to have first line with fieldnames.
+    DEPRECATED Streams lines from an XML file(s)
+    NB assumed is that lines in the file have newlines !!
+    DEPRECATED better is to use XmlElementStreamerFileInput for GML features.
 
-    produces=FORMAT.record
+    produces=FORMAT.xml_line_stream
     """
 
     # Constructor
     def __init__(self, configdict, section):
-        FileInput.__init__(self, configdict, section, produces=FORMAT.record)
+        FileInput.__init__(self, configdict, section, produces=FORMAT.xml_line_stream)
+        self.file_list_done = []
+        self.file = None
+
+    def read(self, packet):
+        # No more files left and done with current file ?
+        if not len(self.file_list) and self.file is None:
+            packet.set_end_of_stream()
+            log.info("EOF file list")
+            return packet
+
+        # Done with current file or first file ?
+        if self.file is None:
+            self.cur_file_path = self.file_list.pop(0)
+            self.file = open(self.cur_file_path, 'r')
+            log.info("file opened : %s" % self.cur_file_path)
+
+        if packet.is_end_of_stream():
+            return packet
+
+        # Assume valid line
+        line = self.file.readline()
+
+        # EOF reached ?
+        if not line or line == '':
+            packet.data = None
+
+            packet.set_end_of_doc()
+            log.info("EOF file")
+            if self.cur_file_path is not None:
+                self.file_list_done.append(self.cur_file_path)
+                self.cur_file_path = None
+                if not len(self.file_list):
+                    # No more files left: end of stream reached
+                    packet.set_end_of_stream()
+                    log.info("EOF file list")
+
+            self.file = None
+
+            return packet
+
+        packet.data = line.decode('utf-8')
+        return packet
+
+
+class CsvFileInput(FileInput):
+    """
+    Parse CSV file into stream of records (dict structures) or a one-time record array.
+    NB raw version: CSV needs to have first line with fieldnames.
+
+    produces=FORMAT.record or FORMAT.record_array
+    """
+
+    @Config(ptype=str, default=',', required=False)
+    def delimiter(self):
+        """
+        A one-character string used to separate fields. It defaults to ','.
+
+        Required: False
+
+        Default: ',' (comma)
+        """
+        pass
+
+
+    @Config(ptype=str, default='"', required=False)
+    def quote_char(self):
+        """
+        A one-character string used to quote fields containing special characters, such as the delimiter or quotechar, or which contain new-line characters. It defaults to '"'
+
+        Required: False
+
+        Default: "
+        """
+        pass
+
+    # Constructor
+    def __init__(self, configdict, section):
+        FileInput.__init__(self, configdict, section, produces=[FORMAT.record_array, FORMAT.record])
         self.file = None
 
     def init(self):
@@ -319,41 +394,60 @@ class CsvFileInput(FileInput):
         log.info('Open CSV file: %s', self.file_path)
         self.file = open(self.file_path)
 
-        self.csv_reader = csv.DictReader(self.file)
+        self.csv_reader = csv.DictReader(self.file, delimiter=self.delimiter, quotechar=self.quote_char)
+
+        if self._output_format == FORMAT.record_array:
+            self.arr = list()
 
     def read(self, packet):
         try:
             packet.data = self.csv_reader.next()
-            log.info("CSV row nr %d read: %s" % (self.csv_reader.line_num-1, packet.data))
+            if self._output_format == FORMAT.record_array:
+                while True:
+                    self.arr.append(packet.data)
+                    packet.data = self.csv_reader.next()
+
+            log.info("CSV row nr %d read: %s" % (self.csv_reader.line_num - 1, packet.data))
         except Exception, e:
+            if self._output_format == FORMAT.record_array:
+                packet.data = self.arr
+
             packet.set_end_of_stream()
             self.file.close()
 
         return packet
 
+
 class JsonFileInput(FileInput):
     """
-    Parse JSON file into hierarchical data struct.
+    Parse JSON file from file system or URL into hierarchical data struct.
+    The struct format may also be a GeoJSON structure. In that case the
+    output_format needs to be explicitly set to geojson_collection in the component
+    config.
 
-    produces=FORMAT.any
+    produces=FORMAT.struct or FORMAT.geojson_collection
     """
 
     # Constructor
     def __init__(self, configdict, section):
-        FileInput.__init__(self, configdict, section, produces=FORMAT.struct)
+        FileInput.__init__(self, configdict, section, produces=[FORMAT.struct, FORMAT.geojson_collection])
 
-    def read(self, packet):
+    def read_file(self, file_path):
+        # One-time read/parse only
         try:
-            log.info('Read JSON file: %s', self.file_path)
             import json
+            # may read/parse JSON from file or URL
+            if file_path.startswith('http'):
+                import urllib2
 
-            with open(self.file_path) as data_file:
-                packet.data = json.load(data_file)
+                fp = urllib2.urlopen(file_path)
+                file_data = json.loads(fp.read())
+            else:
+                with open(file_path) as data_file:
+                    file_data = json.load(data_file)
 
         except Exception, e:
-            log.error('Cannot read JSON file, err= %s', str(e))
+            log.error('Cannot read JSON from %s, err= %s' % (file_path, str(e)))
             raise e
 
-        packet.set_end_of_stream()
-
-        return packet
+        return file_data
